-------------------------------------------------------------------------------
-- Name: class File
-- Author: Jonathan Primeau
--
-- Description:
--     This class allows low level file access. For example, it takes care of
--     opening a file and extract its content. Everytime a method is called, it
--     checks that the appropriate instance of Fileio or FileXtra exists.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pFileIO               -- Fileio Xtra instance
property pFileXtra             -- FileXtra isntance


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  me.CloseFile()
  me.pFileIO = void
  me.pFileXtra = void
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- bool FileExists (me, theFilename)
--
-- Checks if a specified file exists and returns true if it does.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): The filename to look for.
--
-- Return:
--     True is the specified file exists.
--
-------------------------------------------------------------------------------
on FileExists (me, theFilename)
  -- We need FileXtra
  me._NeedFileXtra()
  
  return(me.pFileXtra.fx_FileExists(theFilename))
end

-------------------------------------------------------------------------------
-- bool FolderExists (me, theFolder)
--
-- Checks if the specified folder exists and returns true if it does.
--
-- Parameters:
--     me (object): this pointer.
--     theFolder (string): The folder to look for.
--
-- Return:
--     True is the specified folder exists.
--
-------------------------------------------------------------------------------
on FolderExists (me, theFolder)
  -- We need FileXtra
  me._NeedFileXtra()
  
  return(me.pFileXtra.fx_FolderExists(theFolder))
end

-------------------------------------------------------------------------------
-- void DeleteFile (me, theFilename)
--
-- Deletes the specified file.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): this pointer.
--
-------------------------------------------------------------------------------
on DeleteFile (me, theFilename)
  -- We need FileXtra
  me._NeedFileXtra()
  
  me.pFileXtra.fx_FileDelete(theFilename)
end

-------------------------------------------------------------------------------
-- void Creates (me, theFilename)
--
-- Created the specified file.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): this pointer.
--
-------------------------------------------------------------------------------
on CreateFile (me, theFilename)
  -- We need FileIO
  me._NeedFileIO()
  
  me.pFileIO.CreateFile(theFilename)
end

-------------------------------------------------------------------------------
-- void OpenFile (me, theFilename, theMode)
--
-- Opens a file using specified mode
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): this pointer.
--     theMode (symbol): The mode (#read, #write, #readWrite).
--
-------------------------------------------------------------------------------
on OpenFile (me, theFilename, theMode)
  -- We need FileIO
  me._NeedFileIO()
  
  lMode = 1
  if (theMode = #readWrite) then
    lMode = 0
  else if (theMode = #write) then
    lMode = 2
  else
    lMode = 1
  end if
  
  lStatus = me.pFileIO.OpenFile(theFilename, lMode)
  return(lStatus)
end

-------------------------------------------------------------------------------
-- string ReadFile (me)
--
-- Reads and returns the entire content of the currently opened file.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The content of the file as a string.
--
-------------------------------------------------------------------------------
on ReadFile (me)
  -- We need FileIO
  me._NeedFileIO()
  
  return(me.pFileIO.ReadFile())
end

-------------------------------------------------------------------------------
-- void WriteString (me, theString)
--
-- Writes a string at the current position in the file.
--
-- Parameters:
--     me (object): this pointer.
--     theString (string): String to be written
--
-------------------------------------------------------------------------------
on WriteString (me, theString)
  -- We need FileIO
  me._NeedFileIO()
  
  me.pFileIO.WriteString(theString)
end

-------------------------------------------------------------------------------
-- void WriteReturn (me)
--
-- Writes a return at the current position in the file. It uses the 
-- appropriate return character according to the platform.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on WriteReturn (me)
  -- We need FileIO
  me._NeedFileIO()
  
  lPlatform = void
  if (the platform contains "windows") then
    lPlatform = #windows
  else if (the platform contains "macintosh") then
    lPlatform = #macintosh
  else
    lPlatform = #unix
  end if
  
  me.pFileIO.WriteReturn(lPlatform)
end

-------------------------------------------------------------------------------
-- void SetPositionBOF (me)
--
-- Moves to the begining of file.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on SetPositionBOF (me)
  -- We need FileIO
  me._NeedFileIO()
  
  me.pFileIO.SetPosition(0)
end

-------------------------------------------------------------------------------
-- void SetPositionEOF (me)
--
-- Moves to the end of file. The legth of the file is used to determine 
-- the EOF.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on SetPositionEOF (me)
  -- We need FileIO
  me._NeedFileIO()
  
  lLength = me.pFileIO.GetLength()
  me.pFileIO.SetPosition(lLength)
end

-------------------------------------------------------------------------------
-- void CloseFile (me)
--
-- Closes the currently opened file
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on CloseFile (me)
  -- We need FileIO
  me._NeedFileIO()
  
  me.pFileIO.CloseFile()
  me.pFileIO = void
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pFileIO = void
  me.pFileXtra = void
end

-------------------------------------------------------------------------------
-- void _NeedFileIO (me)
--
-- Create a FileIO Xtra instance if needed.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _NeedFileIO (me)
  if (objectP(me.pFileIO) = 0) then
    me.pFileIO = xtra("fileio").new()
    if (objectP(me.pFileIO) = 0) then ShowMessage("Error creating FileIO")
  end if
end

-------------------------------------------------------------------------------
-- void _NeedFileIO (me)
--
-- Create a FileXtra instance if needed.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _NeedFileXtra (me)
  if (objectP(me.pFileXtra) = 0) then
    me.pFileXtra = xtra("FileXtra4").new()
    if (objectP(me.pFileXtra) = 0) then ShowMessage("Error creating FileXtra")
  end if
end

