-------------------------------------------------------------------------------
-- Name: class InternetList
-- Author: Jonathan Primeau
--
-- Description:
--     This class inherates from ItemList. It represents the Internet site
--     list.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pInternetXML
property pCurrAdTopFile
property pAdTopFile
property pCurrAdList
property pAdList

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class ItemList").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on SetInternetXML (me, theXML)
  me.pInternetXML = theXML
end

on Load (me)
  GenLog(#debug, "InternetList::Load: Loading internet list...")
  
  if (me.pInternetXML = "") then
    GenLog(#error, "InternetList::Banner: Empty XML!")
    SvrLog("URL: Received an empty XML")
    return(-1)
  end if
  
  -- Clear old data
  ancestor.ClearList()
  me.pAdList = []
  
  -- Load and parse XML
  me.LoadXMLFromString(me.pInternetXML)
  lStatus = me.Parse(#_XMLCallback)
  if (lStatus <> 0) then
    me.RevertData()
    return(lStatus)
  end if
  
  if (me.GetCategoryCount() < 2 or me.GetCategoryCount() > 8) then
    GenLog(#error, "InternetList::Load: The number of categories should between 2 and 8!")
    SvrLog("URL: The number of categories should between 2 and 8!")
    me.RevertData()
    return(-2)
  end if
  
  -- Check if ads exist
  lStatus = me._CheckAds()
  if (lStatus = 0) then
    me.pCurrAdList = duplicate(me.pAdList)
  else
    me.RevertData()
    return(-3)
  end if
  
  GenLog(#debug, "InternetList::Load: Internet list loaded!")
  return(0)
end

on GetLineInfo (me, theLineIndex)
  lLineInfo = ancestor.GetLineInfo(theLineIndex)
  
  -- Take what we need from the line info
  lInfo = [:]
  if (lLineInfo.getaProp(#url) <> void) then
    lInfo.addProp(#url, lLineInfo.url)
  end if
  
  return(lInfo)
end

on DisplayInternet (me)
  -- Reload and display the internet section
  GenLog(#debug, "InternetList::DisplayInternet: Displaying internet section...")  
  
  -- Get latest information
  me.UpdateData()
  
  -- We want the view all
  me.ResetTab()
  
  -- Display
  me.DisplayTabs()
  me.Display()
  me._DisplayAds()
  
  GenLog(#debug, "InternetList::DisplayInternet: internet section displayed!")  
end

on SetTab (me, theIndex)
  ancestor.SetTab(theIndex)
  
  me._DisplayAds()
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pMaxCategory = 7
  me.pItemPerPage = 10
  
  me.pLineSpritePrefix = "internet,site,"
  me.pTabSpritePrefix = "internet,categories_tab,"
  me.pListName = "internet"
  
  me.pFieldList.add("name")
  me.pFieldList.add("description")
  
  me.pInternetXML = ""
  
  me.pAdList = []
  me.pCurrAdList = []
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the XML processing because it knows what it has to look for. It is specified
-- to the XML Parse method and is called everytime it encounters an XML tag.
-- If you need to retrieve information from a new tag, you just have to add
-- the symbol corresponding and use the appropriate path. You can easily
-- retrieve any data you want using GetItemValue from the XML parser. Then you
-- have to add the retieved information to the appropriate list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #ad_definition:
      lCategory = string(me.GetAttribute(theItemValue, "category"))
      lAdTopFile = me.GetAttribute(theItemValue, "ad_top")
      lAdBottomFile = me.GetAttribute(theItemValue, "ad_bottom")
      
      lAdFolder = the moviePath & "Ads\"
      
      if (lAdTopFile <> "" and lAdTopFile <> "None") then
        lAdTopFile = lAdFolder & lAdTopFile
      else
        lAdTopFile = lAdFolder & "empty.png"
      end if
      
      if (lAdBottomFile <> "" and lAdBottomFile <> "None") then
        lAdBottomFile = lAdFolder & lAdBottomFile
      else
        lAdBottomFile = lAdFolder & "empty.png"
      end if
      
      lAd = [:]
      lAd.addProp(#category, lCategory)
      lAd.addProp(#adTop, lAdTopFile)
      lAd.addProp(#adBottom, lAdBottomFile)
      
      me.pAdList.add(lAd)
      
    #site:
      -- Get site information
      lName = me.GetItemValue(theItemValue, "name")
      lDescription = me.GetItemValue(theItemValue, "description")
      lCategory = me.GetItemValue(theItemValue, "category")
      lURL = me.GetItemValue(theItemValue, "url")
      
      lCategoryIndex = me._GetCategoryIndex(lCategory)
      if (lCategoryIndex = 0) then
        lCategoryIndex = me.AddCategory(lCategory)
      end if
      
      lCurrentSite = [:]
      lCurrentSite.addProp(#id, me.GetCount() + 1)
      lCurrentSite.addProp(#name, lName)
      lCurrentSite.addProp(#description, lDescription)
      lCurrentSite.addProp(#url, lURL)
      
      me.AddItemToCategory(lCategoryIndex, lCurrentSite)
      me.AddItemToAll(lCategoryIndex, lCurrentSite)
  end case
end

on _CheckAds (me)
  lAdCount = me.pAdList.count
  
  repeat with i = 1 to lAdCount
    lAdTopFile = me.pAdList[i].adTop
    lAdBottomFile = me.pAdList[i].adBottom
    
    if (QI(#file).FileExists(lAdTopFile) = false) then
      GenLog(#error, "InternetList::Load: The following top ad doesn't exist:" && lAdTopFile)
      SvrLog("URL: The following top ad doesn't exist:" && lAdTopFile)
      return(-1)
    end if
    
    if (QI(#file).FileExists(lAdBottomFile) = false) then
      GenLog(#error, "InternetList::Load: The following bottom ad doesn't exist:" && lAdBottomFile)
      SvrLog("URL: The following bottom ad doesn't exist:" && lAdBottomFile)
      return(-2)
    end if
  end repeat
  
  return(0)
end

on _DisplayAds (me)
  lCategoryName = me.GetDisplayedCategoryName()
  
  lAdCount = me.pCurrAdList.count
  repeat with i = 1 to lAdCount
    lCategory = me.pCurrAdList[i].category
    lAdTopFile = me.pCurrAdList[i].adTop
    lAdBottomFile = me.pCurrAdList[i].adBottom
    
    if (lCategoryName = lCategory) then
      sprite("ad_top").member.filename = lAdTopFile
      sprite("ad_bottom").member.filename = lAdBottomFile
      return()
    end if
  end repeat
end
