-------------------------------------------------------------------------------
-- Name: class ItemList
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of displayed list of items. It loads the data 
--     from XML, it keeps the data in memory and displays it on the specified
--     grid.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent

property pCategoryList         -- Category list received from server
property pCategoryItemList     -- Category item list received from server
property pAllItemList          -- All item list received from server

property pCurrCategoryList     -- Category list currently used
property pCurrCategoryItemList -- Category item list currently used
property pCurrAllItemList      -- All item list currently used

property pCurrentPage          -- Current page displayed
property pCurrentTab           -- Current tab index selected
property pMaxCategory          -- Maximum number of categories
property pItemPerPage          -- Number of items per page

property pLineSpritePrefix
property pTabSpritePrefix
property pListName
property pFieldList

property pSelectedLine
property pSelectedIndex
property pSelectedID

-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on DisplayTabs (me)
  GenLog(#debug, "ItemList::DisplayTabs: Loading tabs...")
  
  lCategoryCount = me.pCurrCategoryList.count
  lCategoryMember = me.GetTabMemberPrefix()
  
  -- Index 0 is always View All
  lLabelSprite = me.pTabSpritePrefix & "text,0" 
  lTabSprite = me.pTabSpritePrefix & "0"
  sprite(lLabelSprite).member.text = "View All"
  
  lTabWidth = sprite(me.pTabSpritePrefix & "0").width
  lTabStart = sprite(me.pTabSpritePrefix & "0").locH
  
  repeat with i = 1 to me.pMaxCategory
    lLabelSprite = me.pTabSpritePrefix & "text,"  & i
    lTabSprite = me.pTabSpritePrefix & i
    
    if (i <= lCategoryCount) then
      lTabStart = lTabStart + lTabWidth + 2
      
      sprite(lLabelSprite).member.text = string(me.pCurrCategoryList[i])
      sprite(lTabSprite).locH = lTabStart
      sprite(lLabelSprite).locH = sprite(lTabSprite).rect.left
      sprite(lLabelSprite).member.width = sprite(lTabSprite).width
    else
      sprite(lLabelSprite).member.text = ""
      sprite(lTabSprite).locH = me._GetHiddenLocH()
      sprite(lLabelSprite).locH = me._GetHiddenLocH()
    end if
  end repeat
  
  GenLog(#debug, "ItemList::DisplayTabs: Tabs loaded!")
end

on ClearList (me)
  me.pCategoryList = []
  me.pCategoryItemList = []
  me.pAllItemList = []
end

on Clear (me)
  GenLog(#debug, "ItemList::Clear: Clearing list...")
  
  me.pCategoryList = []
  me.pCategoryItemList = []
  me.pAllItemList = []
  
  me.pCurrCategoryList = []
  me.pCurrCategoryItemList = []
  me.pCurrAllItemList = []
  
  me.pCurrentPage = 1
  me.pCurrentTab = 0
  
  GenLog(#debug, "ItemList::Clear: List cleared!")
end

on Display (me)
  GenLog(#debug, "ItemList::Display: Displaying list...")
  
  lItemList = me._GetDisplayedList()
  
  -- Calculate the starting index
  lStartIndex = me.pCurrentPage * me.pItemPerPage - (me.pItemPerPage - 1)
  
  lItemCount = lItemList.count
  
  repeat with i = 1 to me.pItemPerPage
    lFieldCount = me.pFieldList.count
    
    repeat with j = 1 to lFieldCount
      lSprite = me.pLineSpritePrefix & me.pFieldList[j] & "," & i
      
      if (lStartIndex <= lItemCount) then
        lSymField = symbol(me.pFieldList[j])
        lInfo = lItemList[lStartIndex]
        sprite(lSprite).member.text = string(lInfo.getProp(lSymField))
      else
        sprite(lSprite).member.text = ""
      end if
    end repeat
    
    lStartIndex = lStartIndex + 1
  end repeat
  
  sprite("pageInfo").member.text = "Page" && me.pCurrentPage && "of" && me._GetPageCount()
  
  GenLog(#debug, "ItemList::Display: List displayed!")
end

on GetLineInfo (me, theLineIndex)
  lItemList = me._GetDisplayedList()
  
  lStartIndex = me.pCurrentPage * me.pItemPerPage - (me.pItemPerPage - 1)
  lIndex = lStartIndex + theLineIndex - 1
  
  lItemCount = lItemList.count
  
  lInfo = [:]
  if (lIndex <= lItemCount) then
    lInfo = lItemList[lIndex]
  end if
  
  return(lInfo)
end

-------------------------------------------------------------------------------
-- void GetCount (me)
--
-- Returns the number of channel for a specified category.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The number of channel
--
-------------------------------------------------------------------------------
on GetCount (me, theCategoryIndex)
  if (theCategoryIndex = void) then
    return(me.pAllItemList.count)
  else
    return(me.pCategoryItemList[theCategoryIndex].count)
  end if
end

on GoNextPage (me)
  if (me._IsNextPageAvailable()) then
    me.pCurrentPage = me.pCurrentPage + 1
    GenLog(#debug, "ItemList::GoNextPage:" && string(me.pCurrentPage))
    me.Display()
  end if
end

on GoPreviousPage (me)
  if (me._IsPreviousPageAvailable()) then
    me.pCurrentPage = me.pCurrentPage - 1
    GenLog(#debug, "ItemList::GoPreviousPage:" && string(me.pCurrentPage))
    me.Display()
  end if
end

on GetTabMemberPrefix (me)
  lCategoryCount = me.pCurrCategoryList.count
  return(me.pListName&",categories_tab_"&lCategoryCount)
end

on ResetTab (me)
  me.pCurrentTab = 0
  me.pCurrentPage = 1
end

on SetTab (me, theTabIndex)
  me.pCurrentTab = theTabIndex
  me.pCurrentPage = 1
  me.Display()
end

on GetTab (me)
  return(me.pCurrentTab)
end

on GetCategoryCount (me)
  return(me.pCategoryList.count)
end

on UpdateData (me)
  me.pCurrCategoryList = duplicate(me.pCategoryList)
  me.pCurrCategoryItemList = duplicate(me.pCategoryItemList)
  me.pCurrAllItemList = duplicate(me.pAllItemList)
end

on RevertData (me)
  me.pCategoryList = duplicate(me.pCurrCategoryList)
  me.pCategoryItemList = duplicate(me.pCurrCategoryItemList)
  me.pAllItemList = duplicate(me.pCurrAllItemList)
end

on AddCategory (me, theCategory)
  lCategoryCount = me.pCategoryList.count
  if (lCategoryCount < me.pMaxCategory) then
    me.pCategoryList.add(theCategory)
    me.pCategoryItemList.add([])
  end if
  return(me.pCategoryList.count)
end

on AddItemToCategory (me, theCategoryIndex, theItem)
  me.pCategoryItemList[theCategoryIndex].add(theItem)
end

on AddItemToAll (me, theCategoryIndex, theItem)
  me.pAllItemList.add(theItem)
end

on AdjustPage (me)
  GenLog(#debug, "ItemList::AdjustPage: Ajusting page...")
  
  me.Display()
  
  GenLog(#debug, "ItemList::AdjustPage: Page adjusted!")
end

on SetSelectedIndex (me, theIndex)
  -- Keep the index
  me.pSelectedIndex = theIndex
  
  -- Convert to line
  me.pSelectedLine = me._ConvertIndexToLine(theIndex)
  
  -- Convert to page
  me.pCurrentPage = me._ConvertIndexToPage(theIndex)
end

on GetSelectedIndex (me)
  return(me.pSelectedIndex)
end

on GoNextIndex (me)
  lItemList = me._GetDisplayedList()
  
  lCount = lItemList.count
  
  if (me.pSelectedIndex >= lCount) then
    me.SetSelectedIndex(1)
  else
    me.SetSelectedIndex(me.pSelectedIndex + 1)
  end if
end

on GoPreviousIndex (me)
  lItemList = me._GetDisplayedList()
  
  lCount = lItemList.count
  
  if (me.pSelectedIndex <= 1) then
    me.SetSelectedIndex(lCount)
  else
    me.SetSelectedIndex(me.pSelectedIndex - 1)
  end if
end

on SetSelectedLine (me, theLine)
  -- Keep the line
  me.pSelectedLine = theLine
  
  -- Convert to index
  me.pSelectedIndex = me._ConvertLineToIndex(theLine)
  
  -- The page should not change in this case
end

on GetSelectedLine (me)
  return(me.pSelectedLine)
end

on SetSelectedID (me, theID)
  me.pSelectedID = theID
  
  -- Convert to index
  lSelectedIndex = me._ConvertIDToIndex(theID)
  me.SetSelectedIndex(lSelectedIndex)
end

on GetSelectedID (me)
  lItemList = me._GetDisplayedList()
  
  lCount = lItemList.count
  
  lID = 0
  if (me.pSelectedIndex > 0 and me.pSelectedIndex <= lCount) then
    lID = lItemList[me.pSelectedIndex].id
  end if
  return(lID)
end

on GetItemID (me, theProp, theValue)
  lItemList = me._GetDisplayedList()
  
  lCount = lItemList.count
  
  repeat with i = 1 to lCount
    lCurrItem = lItemList[i]
    if (lCurrItem.getaProp(theProp) <> void) then
      if (lCurrItem.getProp(theProp) = theValue) then
        if (lCurrItem.getaProp(#id) <> void) then
          return(lCurrItem.getProp(#id))
        end if
      end if
    end if
  end repeat
  return(0)
end

on GetDisplayedCategoryName (me)
  -- Tab zero is view all
  if (me.pCurrentTab > 0) then
    return(me.pCurrCategoryList[me.pCurrentTab])
  else
    return("_All_")
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  -- Default value
  me.pMaxCategory = 1 
  me.pItemPerPage = 1 
  
  me.pLineSpritePrefix = ""
  me.pTabSpritePrefix = ""
  me.pListName = "default"
  me.pFieldList = []
  
  me.pCategoryList = []
  me.pCategoryItemList = []
  me.pAllItemList = []
  
  me.pCurrCategoryList = []
  me.pCurrCategoryItemList = []
  me.pCurrAllItemList = []
  
  me.pCurrentPage = 1
  me.pCurrentTab = 0
  
  me.pSelectedLine = 0
  me.pSelectedIndex = 0
  me.pSelectedID = 0
end

on _GetCategoryIndex (me, theCategory)
  lCategoryCount = me.pCategoryList.count
  repeat with i = 1 to lCategoryCount
    if (me.pCategoryList[i] = theCategory) then
      return(i)
    end if
  end repeat
  return(0)
end

on _IsNextPageAvailable (me)
  lItemList = me._GetDisplayedList()
  lCount = lItemList.count
  
  lLastIndex = me.pCurrentPage * me.pItemPerPage
  
  if (lCount > lLastIndex) then
    return(true)
  end if
  return(false)
end

on _IsPreviousPageAvailable (me)
  if (me.pCurrentPage > 1) then
    return(true)
  end if
  return(false)
end

on _GetHiddenLocH (me)
  return(-500)
end

on _ConvertIndexToLine (me, theIndex)
  lLine = theIndex mod me.pItemPerPage
  if (lLine = 0) then lLine = me.pItemPerPage
  return (lLine)
end

on _ConvertIndexToPage (me, theIndex)
  lDivide = me.pItemPerPage
  if (lDivide = 0) then lDivide = 1
  lPage = (theIndex-1) / lDivide + 1
  return (lPage)
end

on _ConvertLineToIndex (me, theLine)
  lStartIndex = me.pCurrentPage * me.pItemPerPage - (me.pItemPerPage - 1)
  lIndex = lStartIndex + theLine - 1
  return(lIndex)
end

on _ConvertIDToIndex (me, theID)
  lItemList = me._GetDisplayedList()
  lCount = lItemList.count
  
  repeat with i = 1 to lCount
    lCurrItem = lItemList[i]
    if (lCurrItem.getProp(#id) = theID) then
      return(i)
    end if
  end repeat
  return(0)
end

on _GetPageCount (me)
  lItemList = me._GetDisplayedList()
  lItemCount = lItemList.count
  
  lDivide = me.pItemPerPage
  if (lDivide = 0) then lDivide = 1
  lTotalPage = (lItemCount - 1) / lDivide + 1
  
  return(lTotalPage)
end

on _GetDisplayedList (me)
  lItemList = []
  if (me.pCurrentTab = 0) then
    lItemList = me.pCurrAllItemList
  else
    lItemList = me.pCurrCategoryItemList[me.pCurrentTab]
  end if
  
  return(lItemList)
end