-------------------------------------------------------------------------------
-- Name: class MUSClient
-- Author: Jonathan Primeau
--
-- Description:
--     This class takes care of the communication between the application and 
--     the servers. This is the low-level side and will use Multiuser Xtra
--     directly.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pConnection           -- Multiuser Xtra instance
property pIsConnected
property pIP                   -- Server IP
property pPort                 -- Server port
property pMessageCallback      -- Callback when getting a message
property pConnectionCallback   -- Callback when connection is made
property pQueue
property pWaitForMoreData
property pWaitBuffer


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
  
  me.Disconnect()
end

on SetIP (me, theIP)
  me.pIP = theIP
end

on SetPort (me, thePort)
  me.pPort = thePort  
end

on SetConnectionCallback (me, theHandler, theObject)
  me.pConnectionCallback.handler = theHandler
  me.pConnectionCallback.object = theObject
end

on SetMessageCallback (me, theHandler, theObject)
  me.pMessageCallback.handler = theHandler
  me.pMessageCallback.object = theObject
end

-------------------------------------------------------------------------------
-- void Connect (me, theMessageHandler, theObject)
--
-- This method connects to the server. It uses the specified handler (and 
-- object) as a callback for any information returned by the server.
--
-- Parameters:
--     me (object): this pointer.
--     theMessageHandler (symbol): The callback for server messages.
--     theObject (object): The object in which the callback resides.
--
-------------------------------------------------------------------------------
on Connect (me)
  me.pIsConnected = false
  
  -- Creates the multiuser instance
  me.pConnection = xtra("Multiuser").new()
  
  -- Set the callback
  lErrCode = me.pConnection.setNetMessageHandler(#_MessageHandler, me)
  
  if (lErrCode <> 0) then
    GenLog(#error, "MUSClient::Connect:" && lErrCode & ": Problem with setNetMessageHandler")
  else
    -- Connect to the server
    GenLog(#debug, "MUSClient2::Connect: Connecting to" && me.pIP & ":" & me.pPort & "...")
    lErrCode = me.pConnection.ConnectToNetServer("IWP", "IWP", me.pIP, me.pPort, "IWP", 1)
    
    if (lErrCode = 0 )then 
      GenLog(#debug, "MUSClient::Connect: OK connectToNetServer sent to server")
    else
      GenLog(#error, "MUSClient::Connect:"&&lErrCode&": Problem sending ConnectToNetServer to server")
      me.pConnection = void
    end if
  end if
end

-------------------------------------------------------------------------------
-- void Disconnect (me)
--
-- This method breaks the connection be deleting the Multiuser Xtra instance.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Disconnect (me)
  -- Break only if there is aready a connection
  if (objectP(me.pConnection)) then
    GenLog(#debug, "MUSClient::Disconnect: Breaking connection...")
    me.pConnection = void
    GenLog(#debug, "MUSClient::Disconnect: OK Connection broken")
    me.pIsConnected = false
  end if
end

on IsConnected (me)
  return(me.pIsConnected)
end

on GetIPAddress (me)
  return(V12OnlineInfos().ip[1])
end

on GetNetAddressCookie ()
  lMUS = xtra("Multiuser").new()
  lCookie = ""
  if (objectP(lMUS)) then
    lOldDelimiter = _player.itemDelimiter
    _player.itemDelimiter = " "
    lCookie = lMUS.GetNetAddressCookie()
    lCookie = lCookie.item[2]
    _player.itemDelimiter = lOldDelimiter
  end if
  lMUS = 0
  return(lCookie)
end

-------------------------------------------------------------------------------
-- void SendCommand (me, theStr)
--
-- This method sends a command to the server.
--
-- Parameters:
--     me (object): this pointer.
--     theStr (string): The command.
--
-------------------------------------------------------------------------------
on SendCommand (me, theStr)
  -- Send only if there is a connection
  if (objectP(me.pConnection)) then   
    -- Send the message
    GenLog(#info, "MUSClient::SendCommand: Sending" && theStr & "...")
    errCode = me.pConnection.sendNetMessage("XML", "String", theStr)
    
    if (errCode = 0) then
      GenLog(#debug, "MUSClient::SendCommand: OK Command sent to server")
    else 
      GenLog(#error, "MUSClient::SendCommand:"&&errCode&": Error with message")
    end if
  else
    GenLog(#error, "MUSClient::SendCommand: Cannot send command, no connection!")
  end if
end

-------------------------------------------------------------------------------
-- list GetNetMessage (me)
--
-- This method returns the message sent by the server. It is meant to be used
-- in the callback (message handler) method. It is public so it can be accessed
-- by external callback.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     A list containing message information
--
-------------------------------------------------------------------------------
on GetNetMessage (me)
  if (objectP(me.pConnection)) then
    return(me.pConnection.getNetMessage())
  end if
  GenLog(#error, "MUSClient::GetNetMessage: Cannot get message, no connection!")
  return("")
end

-------------------------------------------------------------------------------
-- string GetNetErrorString (me, theErrorCode)
--
-- This method returns the net error string corresponding to the error code.
--
-- Parameters:
--     me (object): this pointer.
--     theErrorCode (integer): The error code.
--
-- Return:
--     The net error string.
--
-------------------------------------------------------------------------------
on GetNetErrorString (me, theErrorCode)
  if (objectP(me.pConnection)) then
    return(me.pConnection.getNetErrorString(theErrorCode))
  end if
  GenLog(#error, "MUSClient::GetNetErrorString: Cannot get error, no connection!")
  return("")
end

on ConsumeAllCommands (me)
  lComCount = me.pQueue.count
  
  GenLog(#debug, "MUSClient2::ConsumeAllCommands: Consuming" && string(lComCount) && "commands...")
  
  if (lComCount > 0) then
    -- Consume all commands
    repeat with i = 1 to lComCount
      lContent = me.pQueue[i]
      call(me.pMessageCallback.handler, me.pMessageCallback.object, lContent)
    end repeat
  else
    GenLog(#error, "MUSClient2::ConsumeAllCommands: Trying to consume empty queue!")
  end if
  
  me.pQueue = []
end

on ConsumeOneCommand (me)
  lComCount = me.pQueue.count
  
  GenLog(#debug, "MUSClient2::ConsumeOneCommand: Consuming one command...")
  
  if (lComCount > 0) then
    -- Consume a command
    lContent = me.pQueue[1]
    call(me.pMessageCallback.handler, me.pMessageCallback.object, lContent)
    me.pQueue.deleteAt(1)
  else
    GenLog(#error, "MUSClient2::ConsumeOneCommand: Trying to consume empty queue!")
  end if
end

on ConsumeXCommands (me, theCount)
  lComCount = me.pQueue.count
  
  GenLog(#debug, "MUSClient2::ConsumeXCommands: Consuming" && theCount && "commands...")
  
  if (lComCount > 0) then
    if (theCount <= lComCount) then
      -- Consume X commands
      repeat with i = 1 to theCount
        lContent = me.pQueue[i]
        call(me.pMessageCallback.handler, me.pMessageCallback.object, lContent)
      end repeat
      
      -- Delete consumed commands
      repeat with i = 1 to theCount
        me.pQueue.deleteAt(1)
      end repeat
    else
      GenLog(#error, "MUSClient2::ConsumeXCommands: Trying to consume more commands than what is in the queue!")
    end if
  else
    GenLog(#error, "MUSClient2::ConsumeXCommands: Trying to consume empty queue!")
  end if
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pConnection = void
  
  -- Default values
  me.pIP = "205.236.16.42"
  me.pPort = 50001
  
  me.pMessageCallback = [#handler: void, #object: void]
  me.pConnectionCallback = [#handler: void, #object: void]
  me.pIsConnected = false
  
  me.pQueue = []
  me.pWaitForMoreData = false
  me.pWaitBuffer = ""
end

-------------------------------------------------------------------------------
-- void _MessageHandler (me)
--
-- This is the default callback (message handler) that receives every message
-- from the server. It calls the specified callback for connection (successful)
-- and also for the content
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _MessageHandler (me)
  -- Get message from message queue
  lNetMsg = me.GetNetMessage()
  
  GenLog(#info, "MUSClient::_MessageHandler: Message from server:" && lNetMsg)
  
  -- Error check
  lErrCode = lNetMsg.errorCode
  
  if (lErrCode = 0) then
    -- Set local variables 
    lSenderID = lNetMsg.senderID
    lSubject = lNetMsg.subject
    lContent = lNetMsg.content
    
    if (lSenderID = "System") then
      -- MUS level messages
      case (lSubject) of
        "connectToNetServer": -- Connection has been made now send the command
          me.pIsConnected = true
          GenLog(#debug, "MUSClient::_MessageHandler: SYSTEM: connectToNetServer")
          call(me.pConnectionCallback.handler, me.pConnectionCallback.object)
      end case
    else
      -- Server level messages
      lContent = SearchReplaceString(lContent, GetCommandCR(), GetDirectorCR())
      lCommandCount = me._QueueCommands(lContent)
      if (lCommandCount > 0) then
        me.ConsumeXCommands(lCommandCount)
      end if
    end if
  else
    if (lSenderID = "System") then
      -- MUS level messages
      case (lSubject) of
        "connectionProblem":
          me.pIsConnected = false
          QI(#fatalError).Show()
          QI(#terminalCom).EndPatron()
          GenLog(#error, "MUSClient::_MessageHandler: ConnectionProblem")
      end case
    else
      -- Log errors
      lErrStr = me.GetNetErrorString(lErrCode)
      GenLog(#error, "MUSClient::_MessageHandler:" && lErrStr)
      if (lErrStr = "There is no current connection") then
        me.pIsConnected = false
        QI(#fatalError).Show()
        QI(#terminalCom).EndPatron()
      end if
    end if
  end if
end

on _QueueCommands (me, theContent)
  lLineCount = theContent.line.count
  lCRCount = 0
  lBeginLine = 1
  lCommandCount = 0
  
  repeat with i = 1 to lLineCount
    lLine = theContent.line[i]
    
    if (lLine = "") then
      lCRCount = lCRCount + 1
      
      if (lCRCount = 2) then
        lEndLine = i - 2
        lCommand = theContent.line[lBeginLine..lEndLine]
        
        GenLog(#debug, "MUSClient2::_QueueCommands: Queueing command...")
        
        if (me.pWaitForMoreData = false) then
          -- Command is complete, add it to the queue
          me.pQueue.add(lCommand)
        else
          -- We were waiting for data, the command contains the end of
          -- a command and should be happened to the wait buffer before
          -- being added to the queue
          lBuffer = me.pWaitBuffer & lCommand
          me.pQueue.add(lBuffer)
          me.pWaitBuffer = ""
        end if
        
        lBeginLine = i + 1
        lCRCount = 0
        lCommandCount = lCommandCount + 1
      end if
    end if
  end repeat
  
  -- Check the last two characters. If the content ends by two RETURN
  -- then we got everything we need. If not, we have to wait for the
  -- next message coming from the server to complete the buffer.
  
  lCharCount = theContent.char.count
  if (theContent.line[lLineCount] = "" and theContent.line[lLineCount-1] = "") then
    -- We got everything we need to process the buffer
    me.pWaitForMoreData = false
    me.pWaitBuffer = ""
    -- Return the number of command to process
    return(lCommandCount)
  end if
  
  -- We are missing some data
  me.pWaitForMoreData = true
  
  -- Append the received data to the buffer
  if (lCommandCount = 0) then
    me.pWaitBuffer = me.pWaitBuffer & theContent
  end if
  
  -- Return the number of command to process
  return(lCommandCount)
end
