-------------------------------------------------------------------------------
-- Name: class Skin
-- Author: Jonathan Primeau
--
-- Description:
--     This class contains skin management. It is used to load a new skin
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pSkinInfo             -- Skin information (from XML)
property pSkinPath             -- Complete path including folder
property pSkinFolder           -- Skin folder

property pNavMenu              -- Navigation menu item list
property pAttractLoop          -- Attract loop section item list
property pLogin                -- Login section item list
property pTryout               -- Tryout section item list
property pOffer                -- Offer section item list
property pHome                 -- Home section item list
property pWagering             -- Wagering section item list
property pVideo                -- Video section item list
property pVideoLive            -- Video live section item list
property pVideoMedium          -- Video medium section item list
property pHandicapping         -- Handicapping section item list
property pAccount              -- Account section item list
property pInternet             -- Internet section item list
property pInternetBrowser      -- Internet browser section item list
property pServices             -- Services section item list
property pTimeout
property pLogout
property pSetupAccount
property pDisabled
property pMaintenance
property pMsgDialog

property pServicesMainButtons
property pServicesSubButtons
property pHandicappingMainButtons
property pHandicappingSubButtons


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  -- We need XML Parser
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void Load (me)
--
-- This method loads a skin. It is the main method where everything is 
-- happening. You need to call this method to load a new skin. It parses the
-- skin XML, validates the information and applies the skin.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Load (me)
  GenLog(#debug, "Skin::Load: Loading skin...")
  
  me.Clear()
  
  if (me.pSkinFolder = "") then
    GenLog(#error, "Skin::Load: Skin folder not specified!")
    SvrLog("Skin: Folder not specified")
    return(-1)
  end if
  
  -- Get the skin full path
  lSkinPath = string("Skins\" & me.pSkinFolder)
  
  me.pSkinPath = the moviePath & lSkinPath
  
  -- Check if the skin path exists
  if (QI(#file).FolderExists(me.pSkinPath) = false) then
    GenLog(#error, "Skin::Load: Skin folder doesn't exist" && me.pSkinPath)
    SvrLog("Skin: Folder doesn't exist" && me.pSkinPath)
    me.pSkinPath = ""
    return(-2)
  end if
  
  lSkinFile = lSkinPath & "skin.xml"
  
  -- Load and parse the XML
  me.LoadXMLFromFile(lSkinFile)
  lStatus = me.Parse(#_XMLCallback)
  if (lStatus <> 0) then
    return(lStatus)
  end if
  
  -- If we have a valid skin we can apply it
  lValidSkin = me._ValidateSkin()
  if (lValidSkin = 0) then
    me._ApplySkin()
  else
    return(lValidSkin)
  end if
  
  GenLog(#debug, "Skin::Load: Skin loaded!")
  return(0)
end

-------------------------------------------------------------------------------
-- void Clear (me)
--
-- This method clears the currently loaded skin. It will erase all the members
-- from the Skin castlib.
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Clear (me)
  me._ClearLists()
  me._EmptySkin()
end

-------------------------------------------------------------------------------
-- void GetServicesMainButtons (me)
--
-- This method returns the main buttons in the Services section.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The main button list of the Services section.
--
-------------------------------------------------------------------------------
on GetServicesMainButtons (me)
  return(me.pServicesMainButtons)
end

-------------------------------------------------------------------------------
-- void GetServicesSubButtons (me)
--
-- This method returns the sub button list of the Services section.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The sub button list of the Services section.
--
-------------------------------------------------------------------------------
on GetServicesSubButtons (me)
  return(me.pServicesSubButtons)
end

-------------------------------------------------------------------------------
-- void GetHandicappingMainButtons (me)
--
-- This method returns the main buttons in the Handicapping section.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The main button list of the Handicapping section.
--
-------------------------------------------------------------------------------
on GetHandicappingMainButtons (me)
  return(me.pHandicappingMainButtons)
end

-------------------------------------------------------------------------------
-- void GetHandicappingSubButtons (me)
--
-- This method returns the sub button list of the Handicapping section.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The sub button list of the Handicapping section.
--
-------------------------------------------------------------------------------
on GetHandicappingSubButtons (me)
  return(me.pHandicappingSubButtons)
end

-------------------------------------------------------------------------------
-- void GetLoginHomeURL (me)
--
-- This method returns he URL of the home page (Home section).
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The home URL.
--
-------------------------------------------------------------------------------
on GetLoginHomeURL (me)
  return(me.pSkinInfo.loginHomeURL)
end

-- void GetTryoutHomeURL (me)
--
-- This method returns he URL of the home page (Home section).
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The home URL.
--
-------------------------------------------------------------------------------
on GetTryoutHomeURL (me)
  return(me.pSkinInfo.tryoutHomeURL)
end

-------------------------------------------------------------------------------
-- void GetHomeURL (me)
--
-- This method returns the URL for account management (Account section).
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     The account URL.
--
-------------------------------------------------------------------------------
on GetAccountURL (me)
  return(me.pSkinInfo.accountURL)
end

-------------------------------------------------------------------------------
-- void SetSkinFolder (me)
--
-- This method sets the skin folder (retrieved from the server).
--
-- Parameters:
--     me (object): this pointer.
--     theFolder (string): The folder name of the skin (ie: SKIN001)
--
-------------------------------------------------------------------------------
on SetSkinFolder (me, theFolder)
  me.pSkinFolder = theFolder & "\"
end

on GetButtonTextColor (me)
  if (me.pSkinInfo.getaProp(#buttonTextColor) <> void) then
    return(string(me.pSkinInfo.buttonTextColor))
  else
    return("#000000")
  end if
end

on GetSecButtonTextColor (me)
  if (me.pSkinInfo.getaProp(#secButtonTextColor) <> void) then
    return(string(me.pSkinInfo.secButtonTextColor))
  else
    return("#000000")
  end if
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me) 
  me.pSkinFolder = ""
  me._ClearLists()
end

on _ClearLists (me)
  me.pSkinInfo = [:]
  
  me.pNavMenu = [:]
  me.pAttractLoop = [:]
  me.pLogin = [:]
  me.pTryout = [:]
  me.pOffer = [:]
  me.pHome = [:]
  me.pWagering = [:]
  me.pVideo = [:]
  me.pVideoLive = [:]
  me.pVideoMedium = [:]
  me.pHandicapping = [:]
  me.pAccount = [:]
  me.pInternet = [:]
  me.pInternetBrowser = [:]
  me.pServices = [:]
  me.pTimeout = [:]
  me.pSetupAccount = [:]
  me.pLogout = [:]
  me.pDisabled = [:]
  me.pMaintenance = [:]
  me.pMsgDialog = [:]
  
  me.pSkinPath = ""
  
  me.pServicesMainButtons = []
  me.pServicesSubButtons = []
  me.pHandicappingMainButtons = []
  me.pHandicappingSubButtons = []
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the skin XML because it knows what it has to look for. It is specified to 
-- the XML Parse method and is called everytime it encounters an XML tag.
-- If you need to retrieve information from a new tag, you just have to add
-- the symbol corresponding and use the appropriate path. You can easily
-- retrieve any data you want using GetItemValue from the XML parser. Then you
-- have to add the retieved information to the appropriate list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
      
    #skin:
      lVersion = me.GetAttribute(theItemValue, "version")
      me.pSkinInfo.addProp(#version, lVersion)
      
    #configuration:
      lName = me.GetItemValue(theItemValue, "name")
      me.pSkinInfo.addProp(#name, lName)
      
    #definitions:
      lButtonTextColor = me.GetItemValue(theItemValue, "button_text_color")
      lSecButtonTextColor = me.GetItemValue(theItemValue, "secondary_button_text_color")
      me.pSkinInfo.addProp(#buttonTextColor, lButtonTextColor)
      me.pSkinInfo.addProp(#secButtonTextColor, lSecButtonTextColor)
      
    #attract_loop:
      me.pAttractLoop = [:]
      me.pAttractLoop.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pAttractLoop.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pAttractLoop.addProp(#footer, me.GetItemValue(theItemValue, "footer"))
      
    #login:
      me.pLogin = [:]
      me.pLogin.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pLogin.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pLogin.addProp(#footer, me.GetItemValue(theItemValue, "footer"))
      
      lButtons = [:]
      lButtons.addProp(#setup_account, me._RetreiveButton(theItemValue, "setup_account"))
      lButtons.addProp(#tryout, me._RetreiveButton(theItemValue, "tryout"))
      me.pLogin.addProp(#buttons, lButtons)
      
    #tryout:
      me.pTryout = [:]
      me.pTryout.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pTryout.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#continue_tryout, me._RetreiveButton(theItemValue, "continue_tryout"))
      lButtons.addProp(#setup_account, me._RetreiveButton(theItemValue, "setup_account"))
      me.pTryout.addProp(#buttons, lButtons)
      
    #offer:
      me.pOffer = [:]
      me.pOffer.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pOffer.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pOffer.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#accept_offer, me._RetreiveButton(theItemValue, "accept_offer"))
      lButtons.addProp(#continue, me._RetreiveButton(theItemValue, "continue"))
      me.pOffer.addProp(#buttons, lButtons)
      
    #nav_menu:
      me.pNavMenu = [:]
      me.pNavMenu.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pNavMenu.addProp(#background_slim, me.GetItemValue(theItemValue, "background_slim"))
      me.pNavMenu.addProp(#ad_banner_background, me.GetItemValue(theItemValue, "ad_banner_background"))
      
      lButtons = [:]
      lButtons.addProp(#home, me._RetreiveButton(theItemValue, "home"))
      lButtons.addProp(#wagering, me._RetreiveButton(theItemValue, "wagering"))
      lButtons.addProp(#video, me._RetreiveButton(theItemValue, "video"))
      lButtons.addProp(#handicapping, me._RetreiveButton(theItemValue, "handicapping"))
      lButtons.addProp(#account, me._RetreiveButton(theItemValue, "account"))
      lButtons.addProp(#internet, me._RetreiveButton(theItemValue, "internet"))
      lButtons.addProp(#services, me._RetreiveButton(theItemValue, "services"))
      lButtons.addProp(#logout, me._RetreiveButton(theItemValue, "logout"))
      me.pNavMenu.addProp(#buttons, lButtons)
      
    #home:
      me.pHome = [:]
      me.pHome.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pSkinInfo.addProp(#loginHomeURL, me.GetItemValue(theItemValue, "login_home_url"))
      me.pSkinInfo.addProp(#tryoutHomeURL, me.GetItemValue(theItemValue, "tryout_home_url"))
      
    #wagering:
      me.pWagering = [:]
      
      lButtons = [:]
      lButtons.addProp(#enterbet_logout, me._RetreiveButton(theItemValue, "enterbet_logout"))
      me.pWagering.addProp(#buttons, lButtons)
      
    #video:
      me.pVideo = [:]
      me.pVideo.addProp(#background, me.GetItemValue(theItemValue, "background"))
      
      lButtons = [:]
      lButtons.addProp(#channel_up, me._RetreiveButton(theItemValue, "channel_up"))
      lButtons.addProp(#channel_down, me._RetreiveButton(theItemValue, "channel_down"))
      lButtons.addProp(#volume_up, me._RetreiveButton(theItemValue, "volume_up"))
      lButtons.addProp(#volume_down, me._RetreiveButton(theItemValue, "volume_down"))
      lButtons.addProp(#wagering, me._RetreiveButton(theItemValue, "wagering"))
      lButtons.addProp(#full_screen, me._RetreiveButton(theItemValue, "full_screen"))
      lButtons.addProp(#larger_view, me._RetreiveButton(theItemValue, "larger_view"))
      lButtons.addProp(#live_event, me._RetreiveButton(theItemValue, "live_event"))
      lButtons.addProp(#channel, me._RetreiveButton(theItemValue, "channel"))
      lButtons.addProp(#page_down, me._RetreiveButton(theItemValue, "page_down"))
      lButtons.addProp(#page_up, me._RetreiveButton(theItemValue, "page_up"))
      lButtons.addProp(#categories_tab_2, me._RetreiveButton(theItemValue, "categories_tab_2"))
      lButtons.addProp(#categories_tab_3, me._RetreiveButton(theItemValue, "categories_tab_3"))
      lButtons.addProp(#categories_tab_4, me._RetreiveButton(theItemValue, "categories_tab_4"))
      lButtons.addProp(#categories_tab_5, me._RetreiveButton(theItemValue, "categories_tab_5"))
      lButtons.addProp(#categories_tab_6, me._RetreiveButton(theItemValue, "categories_tab_6"))
      lButtons.addProp(#categories_tab_7, me._RetreiveButton(theItemValue, "categories_tab_7"))
      lButtons.addProp(#categories_tab_8, me._RetreiveButton(theItemValue, "categories_tab_8"))
      me.pVideo.addProp(#buttons, lButtons)
      
      me.pVideo.addProp(#volume_0, me.GetItemValue(theItemValue, "volume_0"))
      me.pVideo.addProp(#volume_1, me.GetItemValue(theItemValue, "volume_1"))
      me.pVideo.addProp(#volume_2, me.GetItemValue(theItemValue, "volume_2"))
      me.pVideo.addProp(#volume_3, me.GetItemValue(theItemValue, "volume_3"))
      me.pVideo.addProp(#volume_4, me.GetItemValue(theItemValue, "volume_4"))
      me.pVideo.addProp(#volume_5, me.GetItemValue(theItemValue, "volume_5"))
      me.pVideo.addProp(#volume_6, me.GetItemValue(theItemValue, "volume_6"))
      
    #video_live:
      me.pVideoLive = [:]
      me.pVideoLive.addProp(#background, me.GetItemValue(theItemValue, "background"))
      
      lButtons = [:]
      lButtons.addProp(#channel_up, me._RetreiveButton(theItemValue, "channel_up"))
      lButtons.addProp(#channel_down, me._RetreiveButton(theItemValue, "channel_down"))
      lButtons.addProp(#volume_up, me._RetreiveButton(theItemValue, "volume_up"))
      lButtons.addProp(#volume_down, me._RetreiveButton(theItemValue, "volume_down"))
      lButtons.addProp(#channel_list, me._RetreiveButton(theItemValue, "channel_list"))
      lButtons.addProp(#full_screen, me._RetreiveButton(theItemValue, "full_screen"))
      lButtons.addProp(#camera_view, me._RetreiveButton(theItemValue, "camera_view"))
      me.pVideoLive.addProp(#buttons, lButtons)
      
      me.pVideoLive.addProp(#volume_0, me.GetItemValue(theItemValue, "volume_0"))
      me.pVideoLive.addProp(#volume_1, me.GetItemValue(theItemValue, "volume_1"))
      me.pVideoLive.addProp(#volume_2, me.GetItemValue(theItemValue, "volume_2"))
      me.pVideoLive.addProp(#volume_3, me.GetItemValue(theItemValue, "volume_3"))
      me.pVideoLive.addProp(#volume_4, me.GetItemValue(theItemValue, "volume_4"))
      me.pVideoLive.addProp(#volume_5, me.GetItemValue(theItemValue, "volume_5"))
      me.pVideoLive.addProp(#volume_6, me.GetItemValue(theItemValue, "volume_6"))
      
      me.pVideoLive.addProp(#camera_1, me.GetItemValue(theItemValue, "camera_1"))
      me.pVideoLive.addProp(#camera_2, me.GetItemValue(theItemValue, "camera_2"))
      me.pVideoLive.addProp(#camera_3, me.GetItemValue(theItemValue, "camera_3"))
      me.pVideoLive.addProp(#camera_4, me.GetItemValue(theItemValue, "camera_4"))
      me.pVideoLive.addProp(#camera_5, me.GetItemValue(theItemValue, "camera_5"))
      me.pVideoLive.addProp(#camera_6, me.GetItemValue(theItemValue, "camera_6"))
      
      me.pVideoLive.addProp(#track_map_0, me.GetItemValue(theItemValue, "track_map_0"))
      me.pVideoLive.addProp(#track_map_1, me.GetItemValue(theItemValue, "track_map_1"))
      me.pVideoLive.addProp(#track_map_2, me.GetItemValue(theItemValue, "track_map_2"))
      me.pVideoLive.addProp(#track_map_3, me.GetItemValue(theItemValue, "track_map_3"))
      me.pVideoLive.addProp(#track_map_4, me.GetItemValue(theItemValue, "track_map_4"))
      me.pVideoLive.addProp(#track_map_5, me.GetItemValue(theItemValue, "track_map_5"))
      me.pVideoLive.addProp(#track_map_6, me.GetItemValue(theItemValue, "track_map_6"))
      
    #video_medium:
      me.pVideoMedium = [:]
      me.pVideoMedium.addProp(#background, me.GetItemValue(theItemValue, "background"))
      
      lButtons = [:]
      lButtons.addProp(#channel_up, me._RetreiveButton(theItemValue, "channel_up"))
      lButtons.addProp(#channel_down, me._RetreiveButton(theItemValue, "channel_down"))
      lButtons.addProp(#volume_up, me._RetreiveButton(theItemValue, "volume_up"))
      lButtons.addProp(#volume_down, me._RetreiveButton(theItemValue, "volume_down"))
      lButtons.addProp(#channel_list, me._RetreiveButton(theItemValue, "channel_list"))
      lButtons.addProp(#full_screen, me._RetreiveButton(theItemValue, "full_screen"))
      lButtons.addProp(#live_event, me._RetreiveButton(theItemValue, "live_event"))
      lButtons.addProp(#wagering, me._RetreiveButton(theItemValue, "wagering"))
      me.pVideoMedium.addProp(#buttons, lButtons)
      
      me.pVideoMedium.addProp(#volume_0, me.GetItemValue(theItemValue, "volume_0"))
      me.pVideoMedium.addProp(#volume_1, me.GetItemValue(theItemValue, "volume_1"))
      me.pVideoMedium.addProp(#volume_2, me.GetItemValue(theItemValue, "volume_2"))
      me.pVideoMedium.addProp(#volume_3, me.GetItemValue(theItemValue, "volume_3"))
      me.pVideoMedium.addProp(#volume_4, me.GetItemValue(theItemValue, "volume_4"))
      me.pVideoMedium.addProp(#volume_5, me.GetItemValue(theItemValue, "volume_5"))
      me.pVideoMedium.addProp(#volume_6, me.GetItemValue(theItemValue, "volume_6"))
      
      me.pVideoMedium.addProp(#sponsor_area, me.GetItemValue(theItemValue, "sponsor_area"))
      
    #handicapping:
      me.pHandicapping = [:]
      me.pHandicapping.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pHandicapping.addProp(#container, me.GetItemValue(theItemValue, "container"))
      
      lButtons = [:]
      lButtons.addProp(#sub_button, me._RetreiveButton(theItemValue, "sub_button"))
      lButtons.addProp(#moreoptions_up, me._RetreiveButton(theItemValue, "moreoptions_up"))
      lButtons.addProp(#moreoptions_down, me._RetreiveButton(theItemValue, "moreoptions_down"))
      me.pHandicapping.addProp(#buttons, lButtons)
      
      -- Dynamic section of handicapping
    #handicapping_main_button:
      if (me.pServicesMainButtons.count <= 3) then
        lButton = [:]
        lButton.addProp(#name, me.GetItemValue(theItemValue, "name"))
        lButton.addProp(#up, me.GetItemValue(theItemValue, "img_up"))
        lButton.addProp(#down, me.GetItemValue(theItemValue, "img_down"))
        lButton.addProp(#text, me.GetItemValue(theItemValue, "text"))
        lButton.addProp(#action, me.GetItemValue(theItemValue, "action"))
        me.pHandicappingMainButtons.add(lButton)
      end if
      
    #handicapping_sub_button:
      lButton = [:]
      lButton.addProp(#name, me.GetItemValue(theItemValue, "name"))
      lButton.addProp(#text, me.GetItemValue(theItemValue, "text"))
      lButton.addProp(#action, me.GetItemValue(theItemValue, "action"))
      me.pHandicappingSubButtons.add(lButton)
      
    #account:
      me.pAccount = [:]
      me.pAccount.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pSkinInfo.addProp(#accountURL, me.GetItemValue(theItemValue, "url"))
      
    #internet:
      me.pInternet = [:]
      me.pInternet.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pInternet.addProp(#sponsor_area, me.GetItemValue(theItemValue, "sponsor_area"))
      me.pInternet.addProp(#ad_space, me.GetItemValue(theItemValue, "ad_space"))
      
      lButtons = [:]
      lButtons.addProp(#site, me._RetreiveButton(theItemValue, "site"))
      lButtons.addProp(#page_down, me._RetreiveButton(theItemValue, "page_down"))
      lButtons.addProp(#page_up, me._RetreiveButton(theItemValue, "page_up"))
      lButtons.addProp(#categories_tab_2, me._RetreiveButton(theItemValue, "categories_tab_2"))
      lButtons.addProp(#categories_tab_3, me._RetreiveButton(theItemValue, "categories_tab_3"))
      lButtons.addProp(#categories_tab_4, me._RetreiveButton(theItemValue, "categories_tab_4"))
      lButtons.addProp(#categories_tab_5, me._RetreiveButton(theItemValue, "categories_tab_5"))
      lButtons.addProp(#categories_tab_6, me._RetreiveButton(theItemValue, "categories_tab_6"))
      lButtons.addProp(#categories_tab_7, me._RetreiveButton(theItemValue, "categories_tab_7"))
      lButtons.addProp(#categories_tab_8, me._RetreiveButton(theItemValue, "categories_tab_8"))
      me.pInternet.addProp(#buttons, lButtons)
      
    #internet_browser:
      me.pInternetBrowser = [:]
      
      lButtons = [:]
      lButtons.addProp(#back, me._RetreiveButton(theItemValue, "back"))
      lButtons.addProp(#forward, me._RetreiveButton(theItemValue, "forward"))
      lButtons.addProp(#scroll_up, me._RetreiveButton(theItemValue, "scroll_up"))
      lButtons.addProp(#scroll_down, me._RetreiveButton(theItemValue, "scroll_down"))
      lButtons.addProp(#scroll_left, me._RetreiveButton(theItemValue, "scroll_left"))
      lButtons.addProp(#scroll_right, me._RetreiveButton(theItemValue, "scroll_right"))
      me.pInternetBrowser.addProp(#buttons, lButtons)
      
    #services:
      me.pServices = [:]
      me.pServices.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pServices.addProp(#container, me.GetItemValue(theItemValue, "container"))
      
      lButtons = [:]
      lButtons.addProp(#sub_button, me._RetreiveButton(theItemValue, "sub_button"))
      lButtons.addProp(#moreoptions_up, me._RetreiveButton(theItemValue, "moreoptions_up"))
      lButtons.addProp(#moreoptions_down, me._RetreiveButton(theItemValue, "moreoptions_down"))
      me.pServices.addProp(#buttons, lButtons)
      
      -- Dynamic section of services
    #services_main_button:
      if (me.pServicesMainButtons.count <= 3) then
        lButton = [:]
        lButton.addProp(#name, me.GetItemValue(theItemValue, "name"))
        lButton.addProp(#up, me.GetItemValue(theItemValue, "img_up"))
        lButton.addProp(#down, me.GetItemValue(theItemValue, "img_down"))
        lButton.addProp(#text, me.GetItemValue(theItemValue, "text"))
        lButton.addProp(#action, me.GetItemValue(theItemValue, "action"))
        me.pServicesMainButtons.add(lButton)
      end if
      
    #services_sub_button:
      lButton = [:]
      lButton.addProp(#name, me.GetItemValue(theItemValue, "name"))
      lButton.addProp(#text, me.GetItemValue(theItemValue, "text"))
      lButton.addProp(#action, me.GetItemValue(theItemValue, "action"))
      me.pServicesSubButtons.add(lButton)
      
    #logout:
      me.pLogout = [:]
      me.pLogout.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pLogout.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pLogout.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
    #setup_account:
      me.pSetupAccount = [:]
      me.pSetupAccount.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pSetupAccount.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#back, me._RetreiveButton(theItemValue, "back"))
      me.pSetupAccount.addProp(#buttons, lButtons)
      
    #disabled:
      me.pDisabled = [:]
      me.pDisabled.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pDisabled.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pDisabled.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#continue, me._RetreiveButton(theItemValue, "continue"))
      me.pDisabled.addProp(#buttons, lButtons)
      
    #maintenance:
      me.pMaintenance = [:]
      me.pMaintenance.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pMaintenance.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pMaintenance.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#admin, me._RetreiveButton(theItemValue, "admin"))
      me.pMaintenance.addProp(#buttons, lButtons)
      
    #timeout:
      me.pTimeout = [:]
      me.pTimeout.addProp(#header, me.GetItemValue(theItemValue, "header"))
      me.pTimeout.addProp(#background, me.GetItemValue(theItemValue, "background"))
      me.pTimeout.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#continue, me._RetreiveButton(theItemValue, "continue"))
      lButtons.addProp(#logout, me._RetreiveButton(theItemValue, "logout"))
      me.pTimeout.addProp(#buttons, lButtons)
      
    #msgdialog:
      me.pMsgDialog = [:]
      me.pMsgDialog.addProp(#window, me.GetItemValue(theItemValue, "window"))
      
      lButtons = [:]
      lButtons.addProp(#ok, me._RetreiveButton(theItemValue, "ok"))
      lButtons.addProp(#cancel, me._RetreiveButton(theItemValue, "cancel"))
      me.pMsgDialog.addProp(#buttons, lButtons)
      
  end case
end

on _RetreiveButton (me, theItemValue, theName)
  lButton = [:]
  lButton.addProp(#type, me.GetItemAttribute(theItemValue, "button", theName, "type"))
  lButton.addProp(#up, me.GetSubItemValue(theItemValue, "button", theName, "img_up"))
  lButton.addProp(#down, me.GetSubItemValue(theItemValue, "button", theName, "img_down"))
  lButton.addProp(#off, me.GetSubItemValue(theItemValue, "button", theName, "img_off"))
  lButton.addProp(#text, me.GetSubItemValue(theItemValue, "button", theName, "text"))
  return lButton
end

-------------------------------------------------------------------------------
-- bool _ValidateSkin (me)
--
-- Validates a skin. This method takes care of validating the entire skin.
-- It calls more specific validation and it returns true if the skin is valid
-- and thus applicable.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     True if all the skin is valid.
-------------------------------------------------------------------------------
on _ValidateSkin (me)
  GenLog(#debug, "Skin::_ValidateSkin: Validating skin...")
  
  lSkinError = 0
  
  -- Validate sections
  if (me._ValidateSection("nav_menu", me.pNavMenu) = false) then lSkinError = -101
  if (me._ValidateSection("attract_loop", me.pAttractLoop) = false) then lSkinError = -102
  if (me._ValidateSection("login", me.pLogin) = false) then lSkinError = -103
  if (me._ValidateSection("tryout", me.pTryout) = false) then lSkinError = -104
  if (me._ValidateSection("setup_account", me.pSetupAccount) = false) then lSkinError = -105
  if (me._ValidateSection("offer", me.pOffer) = false) then lSkinError = -106
  if (me._ValidateSection("home", me.pHome) = false) then lSkinError = -107
  if (me._ValidateSection("wagering", me.pWagering) = false) then lSkinError = -108
  if (me._ValidateSection("video", me.pVideo) = false) then lSkinError = -109
  if (me._ValidateSection("video_live", me.pVideoLive) = false) then lSkinError = -110
  if (me._ValidateSection("video_medium", me.pVideoMedium) = false) then lSkinError = -111
  if (me._ValidateSection("handicapping", me.pHandicapping) = false) then lSkinError = -112
  if (me._ValidateSection("account", me.pAccount) = false) then lSkinError = -1123
  if (me._ValidateSection("internet", me.pInternet) = false) then lSkinError = -114
  if (me._ValidateSection("internet_browser", me.pInternetbrowser) = false) then lSkinError = -115
  if (me._ValidateSection("services", me.pServices) = false) then lSkinError = -116
  if (me._ValidateSection("timeout", me.pTimeout) = false) then lSkinError = -117
  if (me._ValidateSection("logout", me.pLogout) = false) then lSkinError = -118
  if (me._ValidateSection("disabled", me.pDisabled) = false) then lSkinError = -119
  if (me._ValidateSection("maintenance", me.pMaintenance) = false) then lSkinError = -120
  if (me._ValidateSection("msgdialog", me.pMsgDialog) = false) then lSkinError = -121
  if (me._ValidateDynamicScreen(me.pServicesMainButtons, "services") = false) then lSkinError = -122
  if (me._ValidateDynamicScreen(me.pHandicappingMainButtons, "handicapping") = false) then lSkinError = -123
  
  GenLog(#debug, "Skin::_ValidateSkin: Skin validated!")
  return(lSkinError)
end

on _ValidateSection (me, theSectionName, theSectionList)
  lSectionOK = true
  
  if (listP(theSectionList)) then
    lItemCount = theSectionList.count
    
    -- For each item in the section
    repeat with i = 1 to lItemCount
      lItemName = string(theSectionList.getPropAt(i))
      lItemValue = theSectionList[i]
      
      -- If the item is a button
      if (lItemName = "buttons") then
        lButtonCount = lItemValue.count
        
        -- For each button in the list
        repeat with j = 1 to lButtonCount
          lButtonName = string(lItemValue.getPropAt(j))
          lButtonValue = lItemValue[j]
          
          lPartCount = lButtonValue.count
          
          if (lButtonValue.type starts "two_states") then
            -- Up state file
            if (me._CheckSkinFile(lButtonValue.up) = false) then
              lSectionOK = false
            end if
            
            -- Down state file
            if (me._CheckSkinFile(lButtonValue.down) = false) then 
              lSectionOK = false
            end if
            
            -- Off state file
            if (lButtonValue.off <> void) then
              if (me._CheckSkinFile(lButtonValue.off) = false) then 
                lSectionOK = false
              end if
            end if
            
            -- Up state member
            lMemberName = theSectionName & "," & lButtonName & ",up"
            if (me._CheckSkinMember(lMemberName) = false) then 
              lSectionOK = false
            end if
            
            -- Down state member
            lMemberName = theSectionName & "," & lButtonName & ",down"
            if (me._CheckSkinMember(lMemberName) = false) then 
              lSectionOK = false
            end if
            
            -- Off state member
            if (lButtonValue.off <> void) then
              lMemberName = theSectionName & "," & lButtonName & ",off"
              if (me._CheckSkinMember(lMemberName) = false) then 
                lSectionOK = false
              end if
            end if
            
            if (lButtonValue.type <> "two_states_no_label") then
              -- Text member
              lMemberName = theSectionName & "," & lButtonName & ",text"
              if (me._CheckSkinMember(lMemberName) = false) then 
                lSectionOK = false
              end if
            end if
          else
            
          end if
        end repeat
      else
        
        -- Check if the file exists
        if (me._CheckSkinFile(lItemValue) = false) then
          lSectionOK = false
        end if
        
        -- Check if the member exists
        lMemberName = theSectionName & "," & lItemName
        if (me._CheckSkinMember(lMemberName) = false) then
          lSectionOK = false
        end if
      end if
      
    end repeat
    
  end if
  return(lSectionOK)
end

on _ValidateDynamicScreen (me, theList, theSection)
  
  -- Validate the main button graphics (maximum of 3)
  lCount = theList.count
  if (lCount > 3) then
    return(false)
  end if
  
  lSectionOK = true
  
  repeat with i = 1 to lCount
    
    -- Up state file
    if (me._CheckSkinFile(theList[i].up) = false) then 
      lSectionOK = false
    end if
    
    -- Down state file
    if (me._CheckSkinFile(theList[i].down) = false) then 
      lSectionOK = false
    end if
    
    -- Up state member
    lMemberName = theSection & ",main_" & i & ",up"
    if (me._CheckSkinMember(lMemberName) = false) then 
      lSectionOK = false
    end if
    
    -- Down state member
    lMemberName = theSection & ",main_" & i & ",down"
    if (me._CheckSkinMember(lMemberName) = false) then 
      lSectionOK = false
    end if
    
  end repeat
  
  return(lSectionOK)
end

on _CheckSkinFile (me, theFile)
  -- Check if the file exists
  lFilename = me.pSkinPath & theFile
  if (QI(#file).FileExists(lFilename) = false) then
    GenLog(#error, "Skin::_CheckSkinFile: Missing file:"&&lFilename)
    SvrLog("Skin: Missing file:"&&lFilename)
    return(false)
  end if
  return(true)
end

on _CheckSkinMember (me, theMemberName)
  -- Check if the member exists
  if (member(theMemberName) = void) then
    GenLog(#error, "Skin::_CheckSkinMember: Missing member:"&&theMemberName)
    SvrLog("Skin: Missing member:"&&theMemberName)
    return(false)
  end if
  return(true)
end

-------------------------------------------------------------------------------
-- void _ApplySkin (me)
--
-- Applies the skin. 
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _ApplySkin (me)
  GenLog(#debug, "Skin::_ApplySkin: Applying skin...")
  
  -- Applying sections
  me._ApplySection("nav_menu", me.pNavMenu)
  me._ApplySection("attract_loop", me.pAttractLoop)
  me._ApplySection("login", me.pLogin)
  me._ApplySection("tryout", me.pTryout)
  me._ApplySection("setup_account", me.pSetupAccount)
  me._ApplySection("offer", me.pOffer)
  me._ApplySection("home", me.pHome)
  me._ApplySection("wagering", me.pWagering)
  me._ApplySection("video", me.pVideo)
  me._ApplySection("video_live", me.pVideoLive)
  me._ApplySection("video_medium", me.pVideoMedium)
  me._ApplySection("handicapping", me.pHandicapping)
  me._ApplySection("account", me.pAccount)
  me._ApplySection("internet", me.pInternet)
  me._ApplySection("internet_browser", me.pInternetbrowser)
  me._ApplySection("services", me.pServices)
  me._ApplySection("timeout", me.pTimeout)
  me._ApplySection("logout", me.pLogout)
  me._ApplySection("disabled", me.pDisabled)
  me._ApplySection("maintenance", me.pMaintenance)
  me._ApplySection("msgdialog", me.pMsgDialog)
  me._ApplyDynamicScreen(me.pServicesMainButtons, "services")
  me._ApplyDynamicScreen(me.pHandicappingMainButtons, "handicapping")
  
  GenLog(#debug, "Skin::_ApplySkin: Skin applied!")
end

on _ApplySection (me, theSectionName, theSectionList)
  
  if (listP(theSectionList)) then
    lItemCount = theSectionList.count
    
    -- For each item in the section
    repeat with i = 1 to lItemCount
      lItemName = string(theSectionList.getPropAt(i))
      lItemValue = theSectionList[i]
      
      -- If the item is a button
      if (lItemName = "buttons") then
        lButtonCount = lItemValue.count
        
        -- For each button in the list
        repeat with j = 1 to lButtonCount
          lButtonName = string(lItemValue.getPropAt(j))
          lButtonValue = lItemValue[j]
          
          lPartCount = lButtonValue.count
          
          if (lButtonValue.type starts "two_states") then
            -- Up state file
            lFilename = me.pSkinPath & lButtonValue.up
            lMemberName = theSectionName & "," & lButtonName & ",up"
            castLib("Skin").member[lMemberName].filename = lFilename
            
            -- Down state file
            if (lButtonValue.down <> void) then
              lFilename = me.pSkinPath & lButtonValue.down
              lMemberName = theSectionName & "," & lButtonName & ",down"
              castLib("Skin").member[lMemberName].filename = lFilename
            end if
            
            -- Off state file
            if (lButtonValue.off <> void) then
              lFilename = me.pSkinPath & lButtonValue.off
              lMemberName = theSectionName & "," & lButtonName & ",off"
              castLib("Skin").member[lMemberName].filename = lFilename
            end if
            
            if (lButtonValue.type <> "two_states_no_label") then
              -- Text member
              lMemberName = theSectionName & "," & lButtonName & ",text"
              castLib("Skin").member[lMemberName].text = string(lButtonValue.text)
              castLib("Skin").member[lMemberName].color = rgb(me.GetButtonTextColor())
            end if
          else
            
          end if
        end repeat
      else
        
        lFilename = me.pSkinPath & lItemValue
        lMemberName = theSectionName & "," & lItemName
        castLib("Skin").member[lMemberName].filename = lFilename
        
      end if
      
    end repeat
    
  end if
end

on _ApplyDynamicScreen (me, theList, theSection)
  -- Load the main button graphics
  lCount = theList.count
  repeat with i = 1 to lCount
    lFilename = me.pSkinPath & theList[i].up
    lMemberName = theSection & ",main_" & i & ",up"
    castLib("Skin").member[lMemberName].filename = lFilename
    
    lFilename = me.pSkinPath & theList[i].down
    lMemberName = theSection & ",main_" & i & ",down"
    castLib("Skin").member[lMemberName].filename = lFilename
  end repeat
end

on _EmptySkin (me)
  repeat with i = 1 to 550
    if (castLib("skin").member[i].type = #bitmap) then
      castLib("skin").member[i].filename = the moviePath & "Skins\" &"empty.png"
    else if (castLib("skin").member[i].type = #text) then
      castLib("skin").member[i].text = ""
    end if
  end repeat
end
