-------------------------------------------------------------------------------
-- Name: class Timeout
-- Author: Jonathan Primeau
--
-- Description:
--     This is the base class for timeout
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
------------------------------------------------------------------------------- 

property ancestor              -- Parent
property pHit                  -- The current hit value
property pHitScale             -- The scale for each hit
property pTimeout              -- Timeout value
property pName                 -- Timeout name
property pHitCallback          -- Occurs on hit
property pTimeoutCallback      -- Occurs on timeout


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
  
  me.Stop()
end

on Start (me)
  -- Make sure that the timeout is stopped
  me.Stop()
  
  if (me.pName = "" or me.pHitScale = 0) then
    return(false)
  end if
  
  -- Start the timeout
  timeout().new(me.pName, me.pHitScale, #_Hit, me)
  return(true)
end

on Stop (me)
  -- Make sure that it is in the timeout list
  if (me._DoesTimeoutExists()) then
    timeout(me.pName).forget()
  end if
end

on SetHitCallback (me, theHandler, theObject)
  me.pHitCallback = [:]
  me.pHitCallback.addProp(#handler, string(theHandler))
  me.pHitCallback.addProp(#object, theObject)
end

on GetHitCallback (me)
  return(me.pHitCallback)
end

on SetTimeoutCallback (me, theHandler, theObject)
  me.pTimeoutCallback = [:]
  me.pTimeoutCallback.addProp(#handler, string(theHandler))
  me.pTimeoutCallback.addProp(#object, theObject)
end

on GetTimeoutCallback (me)
  return(me.pTimeoutCallback)
end

on SetName (me, theName)
  me.pName = string(theName)
end

on GetName (me)
  return(me.pName)
end

on SetHitScale (me, theScale)
  me.pHitScale = theScale -- in miliseconds
end

on GetHitScale (me)
  return(me.pHitScale) -- in miliseconds
end

on SetTimeout (me, theTimeout)
  me.pTimeout = theTimeout -- in miliseconds
end

on GetTimeout (me)
  return(me.pTimeout) -- in miliseconds
end

on ResetHit (me)
  me.pHit = 0
end

on GetHit (me)
  return(me.pHit)
end

on GetCountDownSec (me)
  if (me.pTimeout <> 0) then
    lCountDownValue = (me.pTimeout - me.pHit) / 1000
  else
    lCountDownValue = 1 -- Infinite
  end if
  return(lCountDownValue)
end

-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pHit = 0
  me.pTimeout = 0
  me.pHitScale = 0
  me.pName = ""
  me.pHitCallback = [#handler: "", #object: void]
  me.pTimeoutCallback = [#handler: "", #object: void]
end

on _Hit (me)
  me.pHit = me.pHit + me.pHitScale
  
  if (me.pHitCallback.handler <> "" and me.pHitCallback.object <> void) then
    call(me.pHitCallback.handler, me.pHitCallback.object)
  end if
  
  lCountDownValue = me.GetCountDownSec()
  
  if (lCountDownValue <= 0) then
    if (me.pTimeoutCallback.handler <> "" and me.pTimeoutCallback.object <> void) then
      call(me.pTimeoutCallback.handler, me.pTimeoutCallback.object)
    end if
  end if
end

on _DoesTimeoutExists (me)
  if (me.pName <> void) then
    lTimeoutCount = _movie.timeoutList.count
    if (lTimeoutCount > 0) then
      repeat with i = 1 to lTimeoutCount
        if (_movie.timeoutList[i] = timeout(me.pName)) then
          return(true)
        end if
      end repeat
    end if
  end if
  return(false)
end