-------------------------------------------------------------------------------
-- Name: class WebProxy
-- Author: Jonathan Primeau
--
-- Description:
--     This class is a web proxy.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pWebProxyXML          -- The Web Proxy XML definition
property pRules                -- The Web Proxy rules
property pRulesBackup          -- If something goes wrong
property pDefaultResponse      -- Default response if no match


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = script("class XMLParser").new()
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

on SetWebProxyXML (me, theXML)
  me.pWebProxyXML = theXML
end

on Load (me)
  GenLog(#debug, "WebProxy::Load: Loading list...")
  
  me.pRules = []
  
  if (me.pWebProxyXML = "") then
    GenLog(#error, "WebProxy::Load: Empty XML!")
    SvrLog("URL Rules: Received an empty XML")
    return(-1)
  end if
  
  -- Load and parse XML
  me.LoadXMLFromString(me.pWebProxyXML)
  lStatus = me.Parse(#_XMLCallback)
  if (lStatus <> 0) then
    -- Problem, use the backup
    me._RevertData()
    return(lStatus)
  end if
  
  -- Problem, use the backup
  me._BackupData()
  
  GenLog(#debug, "VideoList::Load: List loaded!")
  return(0)
end

on Try (me, theURL)
  lCount = me.pRules.count
  
  repeat with i = 1 to lCount
    -- Get current rule properties
    lText = me.pRules[i].text
    lNegative = me.pRules[i].negative
    lResponse = me.pRules[i].response
    
    -- Evaluate using regular expression
    lMatch = EvaluateRegExp(lText, theURL)
    
    if (lMatch = true and lNegative = 0) then
      return(lResponse)
    else if (lMatch = false and lNegative = 1) then
      return(lResponse)
    end if
  end repeat
  return(me.pDefaultResponse)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pWebProxyXML = ""
  me.pRules = []
  me.pRulesBackup = []
  me.pDefaultResponse = "MozillaXtra"
end

on _BackupData (me)
  me.pRulesBackup = duplicate(me.pRules)
end

on _RevertData (me)
  me.pRules = duplicate(me.pRulesBackup)
end

-------------------------------------------------------------------------------
-- void _XMLCallback (me, theItemName, theItemValue)
--
-- Callback used by the XML parser to return data. This method is the brain of
-- the configuration XML because it knows what it has to look for. 
-- It is specified to the XML Parse method and is called everytime it 
-- encounters an XML tag. If you need to retrieve information from a new tag, 
-- you just have to add the symbol corresponding and use the appropriate path. 
-- You can easily retrieve any data you want using GetItemValue from the XML 
-- parser. Then you have to add the retieved information to the appropriate 
-- list.
--
-- Parameters:
--     me (object): this pointer.
--     theItemName (symbol): The current item name being parsed.
--     theItemValue (list): Contains the information for the current item.
--
-------------------------------------------------------------------------------
on _XMLCallback (me, theItemName, theItemValue)
  case (theItemName) of
    #rule:
      lText = me.GetItemValue(theItemValue, "text")
      lType = me.GetItemValue(theItemValue, "type")
      lNegative = me.GetItemValue(theItemValue, "negative")
      lResponse = me.GetItemValue(theItemValue, "response")
      
      lCurrentRule = [:]
      lCurrentRule.addProp(#text, string(lText))
      lCurrentRule.addProp(#type, lType)
      lCurrentRule.addProp(#negative, integer(lNegative))
      lCurrentRule.addProp(#response, string(lResponse))
      me.pRules.add(lCurrentRule)
  end case
end