-------------------------------------------------------------------------------
-- Name: class XMLParser
-- Author: Jonathan Primeau
--
-- Description:
--     This class is high level API over the XML parser (xtra). It is easy 
--     to use. You only need to load an XML string and tell it to parse and it
--     will call the specified callback every times it encounters a tag.
--
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- P R O P E R T I E S
-------------------------------------------------------------------------------

property ancestor              -- Parent
property pFileObj              -- class File script instance
property pItemList             -- The item list (parsed from XML)
property pXmlStr               -- The XML string


-------------------------------------------------------------------------------
-- P U B L I C  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- me new (me)
--
-- This the constructor. It creates the ancestor object (if any) and calls
-- _Init to intialize stuff.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     me (object): A new instance of this class.
--
-------------------------------------------------------------------------------
on new (me)
  me.ancestor = void
  
  -- Initialize
  me._Init()
  
  return(me)
end

-------------------------------------------------------------------------------
-- void Destroy (me)
--
-- This the destructor. It clears everything and also destroys the ancestor
-- (if any.)
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on Destroy (me)
  if (objectP(me.pFileObj)) then
    me.FileObj = void
  end if
  
  if (objectP(me.ancestor)) then
    me.ancestor.Destroy()
  end if
  me.ancestor = void
end

-------------------------------------------------------------------------------
-- void LoadXMLFromFile (me, theFilename)
--
-- This method loads XML from a file. Then it calls LoadXMLFromString using the
-- file content.
--
-- Parameters:
--     me (object): this pointer.
--     theFilename (string): The filename that contains the XML
--
-------------------------------------------------------------------------------
on LoadXMLFromFile (me, thePath)
  
  xmlStr = ""
  lFilename = the moviePath & thePath
  
  -- TODO: Check if the file exists
  lError = QI(#file).OpenFile(lFilename, #read)
  if (lError = 0) then
    xmlStr = QI(#file).ReadFile()
    QI(#file).CloseFile()
  end if
  
  me.LoadXMLFromString(xmlStr)
end

-------------------------------------------------------------------------------
-- void LoadXMLFromString (me, theString)
--
-- This method loads XML from a string. In fact, it only sets pXmlStr, a 
-- property that will be used by the parser later.
--
-- Parameters:
--     me (object): this pointer.
--     theString (string): The XML.
--
-------------------------------------------------------------------------------
on LoadXMLFromString (me, theString)
  lContent = SearchReplaceString(theString, GetDirectorCR(), GetCommandCR())
  me.pXmlStr = lContent
end

-------------------------------------------------------------------------------
-- void Parse (me, theCallback)
--
-- This method parses the XML. You can call this method once the XML string has
-- been loaded.
--
-- Parameters:
--     me (object): this pointer.
--     theCallback (symbol): The callback to call when a tag is encountered.
--
-------------------------------------------------------------------------------
on Parse (me, theCallback)
  lStatus = me._StartParsing()
  if (lStatus <> 0) then
    return(lStatus)
  end if
  if (listP(me.pItemList)) then
    me._ParseItemList(theCallback, me.pItemList)
  else
    GenLog(#error, "XMLParser::Parse: XML parsing failed")
  end if
  return(0)
end

-------------------------------------------------------------------------------
-- list GetItemList (me)
--
-- This method returns a list of parsed items. This is useful only if you need
-- the details of what has been parsed. It is easier to acquire the content 
-- using the callback passed to the Parse method.
--
-- Parameters:
--     me (object): this pointer.
--
-- Return:
--     A list of parsed items.
--
-------------------------------------------------------------------------------
on GetItemList (me)
  return(me.pItemList)
end

-------------------------------------------------------------------------------
-- value GetPathItemValue (me, theItemValue, thePath)
--
-- This method returns an item value using a multilevel list and a path to the
-- required value. It is often called from the callback to retrieve a value from
-- a XML tag.
-- For example, if you pass a list like the following:
--   [#a: [#name "A", #child: [#a1: "A1", #a2: "A2"]]]
-- you can retrieve a1 value by using the following path:
--   "a.child.a1"
--
-- Parameters:
--     me (object): this pointer.
--     theItemValue: A list containg values.
--     thePath: The path to the value required within theItemValue list.
--
-- Return:
--     The value pointed by the path or void if it the path is invalid.
--
-------------------------------------------------------------------------------
on GetPathItemValue (me, theList, thePath)
  
  if (listP(theList) and stringP(thePath)) then
    lOldDelimiter = _player.itemDelimiter
    _player.itemDelimiter = "."
    
    lItemCount = thePath.items.count
    
    lCurrProp = theList
    repeat with i = 1 to lItemCount
      lCurrItem = symbol(thePath.item[i])
      
      if (lCurrProp.getaProp(lCurrItem) = void) then
        _player.itemDelimiter = lOldDelimiter
        return(void)
      end if
      
      lCurrProp = lCurrProp.getProp(lCurrItem)
    end repeat
    
    _player.itemDelimiter = lOldDelimiter
    return(lCurrProp)
  end if
  return(void)
end

on GetAttribute (me, theList, theAttribute)
  if (listP(theList) and stringP(theAttribute)) then
    lOldDelimiter = _player.itemDelimiter
    _player.itemDelimiter = "."
    
    lCurrItem = symbol(theAttribute)
    
    if (theList.getaProp(lCurrItem) = void) then
      _player.itemDelimiter = lOldDelimiter
      return(void)
    end if
    
    _player.itemDelimiter = lOldDelimiter
    return(theList.getProp(lCurrItem))
  end if
  return(void)
end

on GetItemValue (me, theList, theItemTag)
  if (listP(theList) and stringP(theItemTag)) then
    lOldDelimiter = _player.itemDelimiter
    _player.itemDelimiter = "."
    
    lPath = "child." & theItemTag & ".child.value"
    lItemCount = lPath.items.count
    
    lCurrProp = theList
    repeat with i = 1 to lItemCount
      lCurrItem = symbol(lPath.item[i])
      
      if (lCurrProp.getaProp(lCurrItem) = void) then
        _player.itemDelimiter = lOldDelimiter
        return(void)
      end if
      
      lCurrProp = lCurrProp.getProp(lCurrItem)
    end repeat
    
    _player.itemDelimiter = lOldDelimiter
    return(lCurrProp)
  end if
  return(void)
end

on GetItemAttribute (me, theList, theItemTag, theItemName, theAttribute)
  if (listP(theList) and stringP(theItemTag) and \
      stringP(theItemName) and stringP(theAttribute)) then
    
    if (theList.getaProp(#child) = void) then
      return(void)
    end if
    
    lSubItem = theList.child
    lSubItemCount = lSubItem.count
    
    repeat with i = 1 to lSubItemCount
      lItemTag = string(lSubItem.getPropAt(i))
      lItemList = lSubItem[i]
      if (lItemTag = theItemTag) then
        lItemName = me.GetAttribute(lItemList, "name")
        if (lItemName = theItemName) then
          lItemAttribute = me.GetAttribute(lItemList, "type")
          return(lItemAttribute)
        end if
      end if
    end repeat
    
  end if
  return(void)
end

on GetSubItemValue (me, theList, theItemTag, theItemName, theSubItemTag)
  if (listP(theList) and stringP(theItemTag) and \
      stringP(theItemName)and stringP(theSubItemTag)) then
    
    if (theList.getaProp(#child) = void) then
      return(void)
    end if
    
    lSubItem = theList.child
    lSubItemCount = lSubItem.count
    
    repeat with i = 1 to lSubItemCount
      lItemTag = string(lSubItem.getPropAt(i))
      lItemList = lSubItem[i]
      if (lItemTag = theItemTag) then
        lItemName = me.GetAttribute(lItemList, "name")
        if (lItemName = theItemName) then
          lSubItemValue = me.GetItemValue(lItemList, theSubItemTag)
          return(lSubItemValue)
        end if
      end if
    end repeat
    
  end if
  return(void)
end


-------------------------------------------------------------------------------
-- P R I V A T E  M E T H O D S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- void _Init (me)
--
-- Initializes private properties. This is called by the constructor (new).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _Init (me)
  me.pItemList = [:]
  me.pXmlStr = ""
end

-------------------------------------------------------------------------------
-- void _StartParsing (me)
--
-- This method starts parsing. Basically, it uses the XMLParser Xtra to get a
-- parser object that will be used by _DoParsing to make a "simple" list of 
-- items reflecting the XML content (pItemList).
--
-- Parameters:
--     me (object): this pointer.
--
-------------------------------------------------------------------------------
on _StartParsing (me)
  if (string(me.pXmlStr) = "") then
    GenLog(#error, "XMLParser::_StartParsing: XML is empty")
    SvrLog("The XML received is empty")
    return(-101)
  end if
  
  -- Create a new instance of XMLParser
  lParserObj = new(xtra "XMLParser")
  
  if (objectP(lParserObj)) then
    -- Parse the string using
    lErrorCode = lParserObj.parseString(me.pXmlStr)
    lErrorString = lParserObj.getError()
    
    if (voidP(lErrorString)) then
      -- Create a list using the XML content
      me.pItemList = me._DoParsing(lParserObj)
    else
      GenLog(#error, "XMLParser::_StartParsing: Invalid XML:" && lErrorString)
      SvrLog("Invalid XML:" && lErrorString)
      return(-102)
    end if
  else
    GenLog(#error, "XMLParser::_StartParsing: Invalid parser object")
    return(-103)
  end if
  
  -- We don't need the XML parser anymore
  lParserObj = 0
  return(0)
end

-------------------------------------------------------------------------------
-- list _DoParsing (me, theParserObj)
--
-- This method does the parsing. In fact, it creates a list of items by using
-- the parser object. It retrieves attributes, values and children from the
-- object. It is a recursive method meaning that it will call itself when 
-- needed (when it encounters a child item). This method is kind of complex so
-- be CAREFUL when modifying
--
-- Parameters:
--     me (object): this pointer.
--     theParser (object): The parser object used to gather information from.
--
-- Return:
--     A property list of the current item being parsed.
--
-------------------------------------------------------------------------------
on _DoParsing (me, theParserObj)
  
  lCurrItemProp = [:]
  
  -- An element can contain attributes
  if (theParserObj.type = #element) then
    
    -- Add attributes and their values (if any)
    lAttrCount = theParserObj.attributeName.count
    if (lAttrCount > 0) then
      repeat with i = 1 to lAttrCount
        attrName = symbol(theParserObj.attributeName[i])
        attrValue = theParserObj.attributeValue[i]
        lCurrItemProp.addProp(attrName, attrValue)
      end repeat
    end if
    
  else if (theParserObj.type = #text) then
    -- Text contains needed information. Add as value
    lCurrItemProp.addProp(#value, theParserObj.text)
  end if
  
  lChildList = [:]
  lChildCount = theParserObj.count(#child)
  
  -- Parse children recursively
  if (lChildCount > 0) then
    repeat with i = 1 to lChildCount
      lChild = me._DoParsing(theParserObj.child[i])
      if (listP(lChild)) then
        lChildName = lChild.getPropAt(1)
        lChildValue = lChild.getProp(lChildName)
        -- If there is no child name, we reached the bottom
        if (lChildName <> void) then
          lChildList.addProp(lChildName, lChildValue)
        else
          lChildList.addProp(#value, lChildValue.value)
        end if
      end if
    end repeat
  end if
  
  lCurrItemProp.addProp(#child, lChildList)
  
  lCurrItem = [:]
  lCurrItem.addProp(symbol(theParserObj.name), lCurrItemProp)
  
  return(lCurrItem)
end

-------------------------------------------------------------------------------
-- void _ParseItemList (me, theCallback, theItemList, thePropIdx)
--
-- This method parses the item list generated by _DoParsing. It calls the 
-- specified callback everytime it encounters a tag. It is a recursive method
-- and it will call itself for every child.
--
-- Parameters:
--     me (object): this pointer.
--     theCallback (symbol): The callback that will be called for every tag.
--     theItemList (list): The list of items to be parsed.
--     thePropIdx (integer): The index of the current item being parsed.
--
-------------------------------------------------------------------------------
on _ParseItemList (me, theCallback, theItemList, thePropIdx)
  
  if (thePropIdx = void) then thePropIdx = 1
  
  -- Get the current object name
  lItemName = theItemList.getPropAt(thePropIdx)
  lItemValue = theItemList.getAt(thePropIdx)
  
  -- Get the number of children
  lChildCount = 0
  if (listP(lItemValue)) then
    if (lItemValue.getaProp(#child) <> void) then
      lChildCount = lItemValue.child.count
    end if 
  end if
  
  -- Callback
  call(theCallback, me, lItemName, lItemValue)
  
  -- Recursively parse the children
  if (lChildCount > 0) then
    repeat with i = 1 to lChildCount
      me._ParseItemList(theCallback, lItemValue.child, i)
    end repeat
  end if
end